; ---------------------------------------------------------------------------------------------------------
; SUBROTINAS DE CONTROLE DE LCD PADRAO HD44780/JHD204A                                 Spinelli 22/10/2008
;
; As funcoes suportam LCDs HD44780/JHD204A configurados somente para interface de 4 bits.
;
; ---------------------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------------------
; SUPORTE:
; 1 - LCD padrao HD44780/JHD204A.
; 2 - Funciona nas familias 16F e 18F.
; ---------------------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------------------
; NOTAS:
;  1 - A macro SR_4MVC pode ser usada para mover strings para o LCD.
;  2 - A macro SET_CURSOR pode ser usada para posicionar o cursor do LCD
; ---------------------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------------------
; REQUISITOS:
; 1 - O pino RW do LCD deve ficar permanentemente aterrado.
; 2 - Os pinos PORTx<7:4> do PIC, definido pelo simbolo "LCD_port", serao reservados para I/O e
;     deverao ser conectados aos pinos D4 a D7 do LCD. 
; 3 - Manter separados o port de I/O (LCD_port) do port de controle (LCD_cntl).
;
; ---------------------------------------------------------------------------------------------------------

; ---------------------------------------------------------------------------------------------------------
; AS SEGUINTES VARIAVEIS DEVEM SER DEFINIDAS NO PROGRAMA:
;
;#define         _PARALLEL_LCD                    ; LCD com conexoes normais
;LCD_R0          RES   1                          ; registrador de trabalho 0
;LCD_R1          RES   1                          ; registrador de trabalho 1
;LCD_R2          RES   1                          ; registrador de trabalho 2
;LCD_COMMAND     RES   1                          ; buffer de comando
;LCD_TRIS_port   equ   TRISB                      ; registrador de controle de LCD_port
;LCD_port        equ   PORTB                      ; nome da porta de dados do LDC (PORTA, PORTB,GPIO, etc)
;LCD_TRIS_cntl   equ   TRISC                      ; registrador de controle de LCD_cntl
;LCD_cntl        equ   PORTC                      ; nome da porta de controle (PORTA, PORTB,GPIO, etc)
;LCD_EN          equ   7                          ; bit ENABLE/DISABLE na porta de controle
;LCD_RS          equ   6                          ; bit RS na porta de controle 
; ---------------------------------------------------------------------------------------------------------

; ---------------------------------------------------------------------------------------------------------
; EXEMPLO 
; Exibir o caracter "X" na linha 1, coluna 8 do LCD:
;               call   SR_4BITS                  ; inicializa o LCD
;               call   SR_LCDRESET               ; garante que o LCD esta limpo
;               movlw  1                         ; linha 1 
;               call   SR_SETLINE                ; estabelece a linha
;               movlw  8                         ; coluna 8
;               call   SR_SETCOL                 ; estabelece a coluna
;               call   SR_setcursor              ; posiciona o cursor em (1,8)
;               movlw  "X"                       ; W = "X"
;               call   SR_SENDW                  ; envia "X" para a posicao (1,8)
;
; ---------------------------------------------------------------------------------------------------------



; ---------------------------------------------------------------------------------------------------------
; LOG DAS ALTERACOES:
;
; 24/07/2014 - Compatibilizacao dos nomes das funcoes com as funcoes da "lcdsr44780" para LCD serial
;              somente para facilitar a programacao.
;              A macro SR_4MVC passa a suportar esta subrotina.
;              A macro SET_CURSOR agora passa a suportar a funcao LCD_SETCURSOR (SR_SETCURSOR).
;
; 29/03/2014 - Retirado o suporte para interface de 8 bits por nunca ser usada.
;              Criacao da funcao LCD_4BITS.
;              As funcoes LCD_RESET4 e LCD_INIT4 foram descontinuadas e agrupadas na nova funcao LCD_4BITS
;              que passa a inicializar automaticamente o LCD apenas com sua chamada, livrando o programador
;              de se lembrar de chamar um monte de funcoes que sempre sao as mesmas.
;
; 04/06/2011 - Inserida a funcao LCD_SETCGRAM para especificar o endereco da CGRAM quando for necessario
;              definir caracteres do usuario.
;
; 01/06/2011 - Para corrigir problemas de posicionamento do cursor acima da coluna 16, foram adicionadas
;              as funcoes LCD_SETLINE e LCD_SETCOL. 
;              A funcao LCD_SETCURSOR foi modificada para suportar as familias 16F e 18F. 
;              A macro LCD_ADDRESS foi descontinuada, devendo ser usada a macro LCD_LINES em sue lugar.
;              Foram incluidos os alias:
;              LCD_LINESET, LCD_LINESET4, LCD_SETCURSOR, LCD_SENDCMD e LCS_SENDCHR 
;              para as funcoes: 
;              LCD_lineset, LCD_lineset4, LCD_setcursor, LCD_PUTCMD  e LCD_put respectivamente.
;
; 17/05/2011 - As funcoes LCD_setcursor, LCD_lineset e LCD_lineset4 foram modificadas para suportar a leitura
;              da tabela LCD_LINES atraves da insrucao TBLRD. A macro LCD_ADDRESS tambem foi alterada.
;
; 21/03/2011 - Foram acrescentados delays no fim das rotinas LCD_INIT e LCD_INIT4 para dar tempo dos
;              comandos emitidos serem executados.
;
; 20/03/2011 - As rotinas STROBE e STROBE4 agora manda o dado carregado em W para o LCD. Isso se deve ao 
;              diagrama de tempo para as operacoes no LCD onde o Chip Select (E) deve ser levado ao nivel 
;              1 antes do dado ser colocado na porta (segundo o datasheet do LCD).
;
; 20/03/2011 - Rotinas de LCD_wait e LCD_wait160 agora usam a subrotina delay_ms e delay_us.
;
;
; ---------------------------------------------------------------------------------------------------------

; ---------------------------------------------------------------------------------------------------------
; FUNCOES USADAS PARA CONTROLAR O FLUXO DE DADOS DO PIC PARA O LCD:
;
;           SR_LINESET
;           LCD_LINESET
;           LCD_lineset............ posiciona o cursor na primeira coluna de uma determinada linha.
;                                   W deve conter o numero da linha.
;
;           SR_SETLINE
;           LCD_SETLINE
;           LCD_setline ........... estabelece o numero da linha antes de chamar a funcao LCD_SETCURSOR.
;
;           SR_SETCOL
;           LCD_SETCOL
;           LCD_setcol ............ estabelece o numero da coluna antes de chamar a funcao LCD_SETCURSOR.
;
;           SR_SETCURSOR
;           LCD_SETCURSOR
;           LCD_setcursor.......... posicionar o cursor em uma linha e coluna (max 2 linhas X 16 colunas)
;                                   W deve ter um valor como 0xLC onde L e a linha e C e a coluna
;
;           SR_LCDCLEAR
;           LCD_CLEAR
;           LCD_clear.............. apaga o conteudo do LCD
;
;           SR_SENDW
;           LCD_put 
;           LCD_SENDW.............. Envia um caracter para a posicao atual do cursor
;                                   W deve conter o caracter a ser enviado
;
;           SR_CURSORHOME
;           LCD_CURSORHOME
;           LCD_cursorhome......... Envia o cursor para o inicio da tela
;
;           SR_SENDCMD
;           LCD_PUTCMD
;           LCD_putcmd............. Envia para o LCD, um comando que esta na variavel LCD_COMMAND
;                                   Esta rotina e usada pelas rotinas internas
;           SR_LCDRESET
;           LCD_RESET
;           LCD_reset.............. Reset do LCD.
;
;
;           LCD_EXEC
;           LCD_exec............... liga e desliga o pino EN da porta de comando do LCD para ler/gravar 
;                                   o dado presente nos pinos <DB7:DB0> da porta de dados.
;
;           LCD_STROBE
;           LCD_strobe............. Alias de LCD_exec
;
;           LCD_WAIT
;           LCD_wait............... Produz retardo de 1 mili segundo para o LCD poder completar a operacao
;
;           LCD_WAIT160
;           LCD_wait160............ Produz retardo de 160 micro segundo para o LCD poder completar a operacao
;
; ---------------------------------------------------------------------------------------------------------
;           MACROS DE APOIO:
;           SR_4MVC................ Para mover uma string para o LCD 
;           SET_CURSOR............. Para posicionar o cursor do LCD
;           LCD_write.............. [DESCONTINUADA] Para mover uma string para o LCD 
;
; ---------------------------------------------------------------------------------------------------------
;
;
LCD_D7  equ   7
LCD_D6  equ   6
LCD_D5  equ   5
LCD_D4  equ   4

; ---------------------------------------------------------------------------------------------------------
; Inicializa automaticamente o LCD                                                               29/03/2014
; ---------------------------------------------------------------------------------------------------------
SR_4BITS
LCD_4BITS
LCD_4bits
   IF __FAMILIA<18
        bsf  STATUS,RP0
   ENDIF
        bcf     LCD_TRIS_port,LCD_D7                   ; pinos de I/O 7:4 como saidas
        bcf     LCD_TRIS_port,LCD_D6                   ; 
        bcf     LCD_TRIS_port,LCD_D5                   ; 
        bcf     LCD_TRIS_port,LCD_D4                   ; 
        bcf     LCD_TRIS_cntl,LCD_EN
        bcf     LCD_TRIS_cntl,LCD_RS
   IF __FAMILIA<18
        bcf  STATUS,RP0
   ENDIF
                 bcf        LCD_cntl,LCD_EN      ; EN=0
                 bcf        LCD_cntl,LCD_RS      ; RS=0
                 movlw      .250
                 call       delay_ms
                 movlw      .250
                 call       delay_ms
                 movlw      .250
                 call       delay_ms
                 movlw      .250
                 call       delay_ms

; ----------- LCD_RESET  ------------------------------------------------
                 movlw      0x0F                       ;    w = 0000 1111
                 andwf      LCD_port,f                 ; port = 0000 xxxx
                 movlw      0x30                       ;    w = 0011 0000
                 call       LCD_STROBE

                 movlw      .5
                 movwf      LCD_R1
_wlcd001         call       LCD_wait
                 decfsz     LCD_R1,f
                 goto       _wlcd001
;
                 movlw      0x0F
                 andwf      LCD_port,f
                 movlw      0x30
                 call       LCD_STROBE
                 call       LCD_wait160
;
                 movlw      0x0F
                 andwf      LCD_port,f
                 movlw      0x30
                 call       LCD_STROBE
                 call       LCD_wait160
;
                 movlw      0x0F
                 andwf      LCD_port,f
                 movlw      0x20
                 call       LCD_STROBE
                 call       LCD_wait160
; ----------------------------------------------------------------
                 movlw      .250
                 call       delay_ms
;
                 movlw      b'00100000'          ; FUNCTIOM CMD
                 movwf      LCD_COMMAND
                 movlw      b'00101111'          ; interface de 4 bits
                 andwf      LCD_COMMAND,f
                 movlw      b'00101000'          ; display de 2 linhas
                 iorwf      LCD_COMMAND,f
                 movlw      b'00111011'          ; matriz 5X7
                 andwf      LCD_COMMAND,f
                 call       LCD_PUTCMD

                 movlw      b'00001000'          ; DISPLAY CMD
                 movwf      LCD_COMMAND
                 movlw      b'00001011'          ; DISPLAY OFF
                 andwf      LCD_COMMAND,f
                 movlw      b'00001101'          ; CURSOR OFF
                 andwf      LCD_COMMAND,f
                 movlw      b'00001110'          ; NO BLINK
                 andwf      LCD_COMMAND,f
                 call       LCD_PUTCMD

                 movlw      b'00001000'          ; DISPLAY CMD
                 movwf      LCD_COMMAND
                 movlw      b'00001100'          ; DISPLAY ON
                 iorwf      LCD_COMMAND,f
                 movlw      b'00001010'          ; CURSOR ON
                 iorwf      LCD_COMMAND,f
                 movlw      b'00001001'          ; BLINK
                 iorwf      LCD_COMMAND,f
                 call       LCD_PUTCMD

                 movlw      b'00000100'          ; ENTRY MODE CMD
                 movwf      LCD_COMMAND
                 movlw      b'00000110'          ; display noshifted
                 andwf      LCD_COMMAND,f
                 movlw      b'00000110'          ; increase
                 iorwf      LCD_COMMAND,f
                 call       LCD_PUTCMD

                 call       LCD_clear
                 call       LCD_PUTCMD
                 return

; ---------------------------------------------------------------------------------------------------------
; LCD_put  - INTERFACE 4 BITS                                                                   27/10/2008
; Envia um caracter para a posicao atual do cursor
; ENTRADA: W deve conter o caracter a ser enviado
; ---------------------------------------------------------------------------------------------------------
SR_SENDW
SR_SENDCHR
LCD_SENDW
LCD_put      bsf     LCD_cntl,LCD_RS     ; RS=1 --> dados
             movwf   LCD_COMMAND         ; salva o dado a enviar
             movlw   0x0F
             andwf   LCD_port,f          ; limpa MSB do port
             movf    LCD_COMMAND,w       ; pega o dado
             andlw   0xF0                ; deixa so o MSB
             call    LCD_STROBE          ; STROBE
             movlw   0x0F
             andwf   LCD_port,f            ; limpa MSB do port
             swapf   LCD_COMMAND,w       ; pega o LSB
             andlw   0xF0                ; deixa so o LSB
             call    LCD_STROBE          ; STROBE
             call    LCD_wait            ; aguarda termino do I/O
             bcf     LCD_cntl,LCD_RS     ; RS=0 --> comandos
             return                      ; fim
;
; ---------------------------------------------------------------------------------------------------------
;            Envia um comando ao LCD EM 4 BITS                                                  27/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_SENDCMD
LCD_PUTCMD
LCD_putcmd   bcf   LCD_cntl,LCD_RS       ; RS=0 --> comando
             movlw 0x0F
             andwf LCD_port,f            ; limpa MSC do port
             movf  LCD_COMMAND,w         ; pega o comando
             andlw 0xF0                  ; deixa so o MSB
             call  LCD_STROBE            ; executa o comando
             movlw 0x0F
             andwf LCD_port,f            ; limpa MS|B do port
             swapf LCD_COMMAND,w         ; pega o LSB
             andlw 0xF0                  ; deixa so o LSB
             call  LCD_STROBE            ; executa o comando
             call  LCD_wait              ; aguarda terminar
             return                      ; fim


; ---------------------------------------------------------------------------------------------------------
; LCD_RESET  - Reset para interface de 4 bits                                                   27/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_LCDRESET
LCD_RESET
LCD_reset    movlw  0x0F                       ;    w = 0000 1111
             andwf  LCD_port,f                 ; port = 0000 xxxx
             movlw  0x30                       ;    w = 0011 0000
             call   LCD_STROBE

             movlw  .5
             movwf  LCD_R1
_wlcd002     call   LCD_wait
             decfsz LCD_R1,f
             goto   _wlcd002

             movlw  0x0F
             andwf  LCD_port,f
             movlw  0x30
             call   LCD_STROBE
             call   LCD_wait160

             movlw  0x0F
             andwf  LCD_port,f
             movlw  0x30
             call   LCD_STROBE
             call   LCD_wait160

             movlw  0x0F
             andwf  LCD_port,f
             movlw  0x20
             call   LCD_STROBE
             call   LCD_wait160
             return


; ---------------------------------------------------------------------------------------------------------
; LCD_exec - liga e desliga o pino EN da porta de comando do LCD                                24/10/2008
;            para ler/gravar o dado presente nos pinos <DB7:DB0>
;            da porta de dados.
;            W contem o caracter a ser enviado.
; ---------------------------------------------------------------------------------------------------------
LCD_STROBE
LCD_strobe
LCD_EXEC
LCD_exec     bsf     LCD_cntl,LCD_EN     ; LCD enable
             nop
             movwf   LCD_port            ; move W para a porta de dados
             nop                         ; tempo
             nop                         ; tempo
             nop                         ; tempo
             bcf     LCD_cntl,LCD_EN     ; LCD disable
             nop                         ; tempo
             nop                         ; tempo
             nop                         ; tempo
             return                      ; fim



; ---------------------------------------------------------------------------------------------------------
; LCD_wait - Aguarda 1mS (mili segundo) para o LCD terminar o comando                           24/10/2008
; ---------------------------------------------------------------------------------------------------------
LCD_WAIT
LCD_wait    movlw     .1
            call      delay_ms
            return
;
; ---------------------------------------------------------------------------------------------------------
; LCD_wait160 - Aguarda 160uS (micro segundos) para o LCD terminar o comando                    27/10/2008
; ---------------------------------------------------------------------------------------------------------
LCD_WAIT160
LCD_wait160  movlw    .160
             call     delay_us
             return
;

; ---------------------------------------------------------------------------------------------------------
; LCD_setcursor - Posiciona o cursor em LCDS 2x16, 4x16 e 4x20                        Spinelli - 01/06/2011
;
; Esta rotina exige que a macro LCD_LINES seja emitida no inicio do programa para gerar a tabela de
; enderecos das linhas do LCD como a seguir:
;
; ENTRADA: LCD_R0 contem o numero da linha
;          LCD_R1 contem o numero da coluna
; 
; SAIDA  : LCD posicionado
;
; CHAMADA:
;            movlw   2                ; linha 2
;            call    LCD_setline       ; estabelece a linha
;            movlw   4                ; coluna 4
;            call    LCD_setcol        ; estabelece a coluna
;            call    LCD_setcursor     ; gera o comando para posicionar cursor
;            call    LCD_SENDCMD       ; executa o comando
;
; ---------------------------------------------------------------------------------------------------------
SR_SETLINE
LCD_SETLINE
LCD_setline     movwf    LCD_R0                  ; salva o numero da linha
                decf     LCD_R0,f                ; ajusta o numero da linha
                return                           ;
;
LCD_SETCOL
LCD_setcol      movwf    LCD_R1                  ; salva o numero da coluna
                decf     LCD_R1,f                ; ajusta o numero da coluna
                return                           ;
;
SR_SETCURSOR
LCD_SETCURSOR
LCD_setcursor   

   IF __FAMILIA==18
             movlw   UPPER(LCD_ADDRTAB)          ; endereco da tabela
             movwf   TBLPTRU                     ; carrega apontador
             movlw   HIGH(LCD_ADDRTAB)           ; endereco da tabela
             movwf   TBLPTRH                     ; carrega apontador
             movlw   LOW(LCD_ADDRTAB)            ; endereco da tabela
             addwf   LCD_R0,w                    ; calcula o deslocamento na tabela
             movwf   TBLPTRL                     ; estabelece o endereco
             tblrd   *+                          ; le o endereco da linha
             movf    TABLAT,w                    ; pega o endereco lido
             addwf   LCD_R1,w                    ; adiciona o numero da coluna
             iorlw   0x80                        ; mistura endereco calculado com o comando
             movwf   LCD_COMMAND                 ; insere comando no buffer
   ENDIF

   IF __FAMILIA==16
             movf    LCD_R0,w                    ; pega o numero da linha
             call    LCD_ADDRTAB                 ; le o endereco na tabela
             addwf   LCD_R1,w                    ; adiciona o numero da coluna
             iorlw   0x80                        ; mistura endereco calculado com o comando
             movwf   LCD_COMMAND                 ; insere comando no buffer
   ENDIF

             call   LCD_PUTCMD 
             return



; ---------------------------------------------------------------------------------------------------------
; LCD_lineset                                                                         Spinelli - 14/02/2009
; Posiciona o cursor na primeira coluna de uma determinada linha.
; Suporta LCDs de ate 4 linhas.
; 
; ENTRADA: W deve conter o numero da linha.
;
; Esta rotina exige o uso da macro LCD_LINES no inicio do programa para montar a tabela de 
; enderecos de linhas do LCD.
; 
; Exemplo:
;            movlw   .2
;            call    LCD_lineset 
;
; ---------------------------------------------------------------------------------------------------------
SR_LINESET
LCD_LINESET 
LCD_lineset  movwf   LCD_R0                      ; numero da linha
             decf    LCD_R0,f
             clrf    LCD_R1                      ; forca a coluna zero
             call    LCD_setcursor
             return


; ---------------------------------------------------------------------------------------------------------
; LCD_clear - Limpa o LCD                                                                       24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_LCDCLEAR
LCD_CLEAR
LCD_clear    movlw   0x01                ; comando 01 - CLEAR
             movwf   LCD_COMMAND         ; armazena o comando
             call    LCD_PUTCMD 
             return                      ; fim
;
; ---------------------------------------------------------------------------------------------------------
; LCD_cursorhome - Envia o cursor para o inicio                                                 24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_CURSORHOME
LCD_CURSORHOME
LCD_cursorhome movlw b'00000010'         ; comando "cursor home"
               movwf   LCD_COMMAND       ; armazena o comando
               call    LCD_PUTCMD 
               return                    ; fim
;



;
; ---------------------------------------------------------------------------------------------------------
;                COMANDO SET CGRAM                                                              04/06/2011
; ENTRADA: W contem o endereco da CGRAM
; ---------------------------------------------------------------------------------------------------------
SR_SETCGRAM
LCD_SETCGRAM
LCD_setcgram     iorlw   0x40                  ; mistura o endereco com o comando
                 movwf    LCD_COMMAND
                 call     SR_SENDCMD
                 movlw    .30
                 call     delay_ms
                 return










; ---------------------------------------------------------------------------------------------------------
;           A INICIALIZACAO DO LCD DEVE SER FEITA COM AS SEGUINTES CHAMADAS COMO NA SEQUENCIA ABAIXO:
;
;           call   LCD_INIT                                              tempo de espera apos power on
;           call   LCD_FUNCTION_CMD                                      registra o comando FUNCTION
;           call   LCD_FUNCTION_4B         ou LCD_FUNCTION_8B            interface de 4 ou 8 bits
;           call   LCD_FUNCTION_1L         ou LCD_FUNCTION_2L            LCD de 1 ou duas linhas
;           call   LCD_FUNCTION_5X7        ou LCD_FUNCTION_5X10          matriz de caracter
;           call   LCD_PUTCMD                                            envia o comando registrado
;
;           call   LCD_DISPLAY_CMD                                       registra o comando DISPLAY
;           call   LCD_DISPLAY_ON          ou LCD_DISPLAY_OFF            liga/desliga LCD
;           call   LCD_DISPLAY_CURSOR_ON   ou LCD_DISPLAY_CURSOR_OFF     liga/desliga corsor
;           call   LCD_DISPLAY_BLINK       ou LCD_DISPLAY_NOBLINK        pisca oou nao pisca
;           call   LCD_PUTCMD                                            envia o comando registrado
;
;           call   LCD_ENTRYMODE_CMD                                     registra o comando ENTRY MODE
;           call   LCD_ENTRYMODE_SHIFT     ou LCD_ENTRYMODE_NOSHIFT      desloca/ nao desloca
;           call   LCD_ENTRYMODE_ID        ou LCD_ENTRYMODE_NOID         avanca/ nao avanca o cursor
;           call   LCD_PUTCMD                                            envia o comando registrado (8 bits)
;
;           call   LCD_SHIFT_CMD                                         registra o comando SHIFT     
;           call   LCD_SHIFT               ou LCD_SHIFT_CMOVE            move cursos/caracter
;           call   LCD_SHIFT_RIGHT         ou LCD_SHIFT_LEFT             para adireita ou esquerda
;           call   LCD_PUTCMD                                            envia o comando registrado
;
; ---------------------------------------------------------------------------------------------------------
;                COMANDO SET FUNCTION                                                           24/10/2008
; ---------------------------------------------------------------------------------------------------------
;LCD_FUNCTION_CMD movlw    b'00100000'         ; FUNCTIOM CMD
;                 movwf    LCD_COMMAND
;                 return
;LCD_FUNCTION_8B  movlw    b'00110000'         ; interface de 8 bits
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_FUNCTION_4B  movlw    b'00101111'         ; interface de 4 bits
;                 andwf    LCD_COMMAND,f
;                 return
;LCD_FUNCTION_2L  movlw    b'00101000'         ; display de 2 linhas
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_FUNCTION_1L  movlw    b'00110111'         ; display de 1 linha
;                 andwf    LCD_COMMAND,f
;                 return
;LCD_FUNCTION_5X10  movlw  b'00100100'         ; matriz 5X10
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_FUNCTION_5X7   movlw  b'00111011'         ; matriz 5X7
;                 andwf    LCD_COMMAND,f
;                 return



; ---------------------------------------------------------------------------------------------------------
;                COMANDO DISPLAY                                                                24/10/2008
; ---------------------------------------------------------------------------------------------------------
;LCD_DISPLAY_CMD  movlw    b'00001000'     ; DISPLAY CMD
;                 movwf    LCD_COMMAND
;                 return
;LCD_DISPLAY_ON   movlw    b'00001100'     ; DISPLAY ON
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_DISPLAY_OFF  movlw    b'00001011'     ; DISPLAY OFF
;                 andwf    LCD_COMMAND,f
;                 return
;LCD_DISPLAY_CURSOR_ON movlw b'00001010'   ; CURSOR ON
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_DISPLAY_CURSOR_OFF movlw b'00001101'  ; CURSOR OFF
;                 andwf    LCD_COMMAND,f
;                 return
;LCD_DISPLAY_BLINK movlw   b'00001001'     ; BLINK
;                 iorwf    LCD_COMMAND,f
;                 return
;LCD_DISPLAY_NOBLINK movlw   b'00001110'   ; NO BLINK
;                 andwf    LCD_COMMAND,f
;                 return



; ---------------------------------------------------------------------------------------------------------
;                COMANDO ENTRY MODE                                                             24/10/2008
; ---------------------------------------------------------------------------------------------------------
;LCD_ENTRYMODE_CMD movlw     b'00000100'        ; ENTRY MODE CMD
;                 movwf      LCD_COMMAND
;                 return
;LCD_ENTRYMODE_SHIFT movlw   b'00000101'        ; display shifted
;                 iorwf      LCD_COMMAND,f
;                 return
;LCD_ENTRYMODE_NOSHIFT movlw b'00000110'        ; display noshifted
;                 andwf      LCD_COMMAND,f
;                 return
;LCD_ENTRYMODE_ID movlw      b'00000110'        ; increase
;                 iorwf      LCD_COMMAND,f
;                 return
;LCD_ENTRYMODE_NOID movlw    b'00000101'         ; decrease
;                 andwf      LCD_COMMAND,f
;                 return
;



; ---------------------------------------------------------------------------------------------------------
;                COMANDO SHIFT                                                                  24/10/2008
; ---------------------------------------------------------------------------------------------------------
;LCD_SHIFT_CMD    movlw     b'00010000'         ; SHIFT COMMAND
;                 movwf     LCD_COMMAND
;                 return
;LCD_SHIFT        movlw     b'00011000'         ; Display shift
;                 iorwf     LCD_COMMAND,f
;                 return
;LCD_SHIFT_CMOVE  movlw     b'00010111'         ; Display shift cursor move
;                 andwf     LCD_COMMAND,f
;                 return
;LCD_SHIFT_RIGHT  movlw     b'00010100'
;                 iorwf     LCD_COMMAND,f       ; Display shift right
;                 return
;LCD_SHIFT_LEFT   movlw     b'00011011'         ; Display shift left
;                 andwf     LCD_COMMAND,f
;                 return




;
; -------------------------------------------------------------------------------------------------------------------------
;              LCD - FORMATO DOS COMANDOS
; -------------------------------------------------------------------------------------------------------------------------
;              RS  RW  DB7  DB6  DB5  DB4  DB3  DB2  DB1  DB0 
; -------------------------------------------------------------------------------------------------------------------------
; DISPLAY      L   L    L    L    L    L    L    L    L    H         
; CLEAR
; -------------------------------------------------------------------------------------------------------------------------
; RETURN       L   L    L    L    L    L    L    L    H    X         move cursor para o digito 1
; HOME
; -------------------------------------------------------------------------------------------------------------------------
; ENTRY        L   L    L    L    L    L    L    H    ID   SH        ID -> H=increase              L=decrease
; MODE                                                               SH -> H=display shifted       L=display not shifted
; -------------------------------------------------------------------------------------------------------------------------
; DISPLAY      L   L    L    L    L    L    H    D    C    B          D -> H=display on            L=display off
; ON/OFF                                                              C -> H=cursor on             L=cursor off
;                                                                     B -> H=blinking on           L=blinking off
; -------------------------------------------------------------------------------------------------------------------------
; SHIFT        L   L    L    L    L    H    SC   RL   X    X         SC -> H=display shift         L=cursor move
;                                                                    RL -> H=right shift           L=left shift
; -------------------------------------------------------------------------------------------------------------------------
; SET          L   L    L    L    H    DL   N    F    X    X         DL -> H=8 bits interface      L=4 bits interface
; FUNCTION                                                            N -> H=display de 2 linhas   L=dsiplay de 1 linha
;                                                                     F -> H=matriz 5X10           L=matriz 5X7
; -------------------------------------------------------------------------------------------------------------------------
; SET CG RAM   L   L    L    H    |<--- CG RAM ADDRESS ---->|        CG RAM data enviado e recebido apos este ordem
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; SET DD RAM   L   L    H    |<----- DD RAM ADDRESS ------->|        DD RAM data enviado e recebido apos este ordem
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; READ BUSY    L   H    BF   |<-- ADDRESS COUNTER USED ---->|        BF -> H=busy                  L=ready
; FLAG &                     | FOR BOTH DD & CG RAM ADDRESS |              BF indica a leitura do status do LCD
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; WRITE DATA   H   L    |<------------ data --------------->|        grava dado em DD ou CG RAM
; -------------------------------------------------------------------------------------------------------------------------
; READ  DATA   H   L    |<------------ data --------------->|        le daoa em DD ou CG RAM
; -------------------------------------------------------------------------------------------------------------------------
;

